/* !!! THIS MODULE HAS AST_MODFLAG_GLOBAL_SYMBOLS !!! */
/* all global variables should be static */
#include <module.h>

ASTERISK_FILE_VERSION(__FILE__, "$Revision$")

#ifdef	__cplusplus
extern "C" {
#endif

#include "activeCallsModule.h"
#include "blacklistModule.h"
#include "callConstraintsModule.h"
#include "callPermissionsModule.h"
#include "dialplanModule.h"
#include "helperModule.h"
#include "historyModule.h"
#include "routingModule.h"
#include "securityModule.h"
#include "timeRangeModule.h"

	void ami_send_message(struct stasis_message_type *type, struct ast_json *json_object)
	{
		RAII_VAR(struct stasis_message *, msg, NULL, ao2_cleanup);
		RAII_VAR(struct ast_json_payload *, json_payload, NULL, ao2_cleanup);

		if (!(json_payload = ast_json_payload_create(json_object))) {
			ast_log(AST_LOG_ERROR, "Failed to create the payload.\n");
			return;
		}

		msg = stasis_message_create(type, json_payload);
		stasis_publish(ast_manager_get_topic(), msg);
	}

	static int process_module(const char *action, const char *module, int (*fnc)())
	{
		int res = 0;
		ast_debug(3, "%s module " AST_MODULE "... (%s)...\n", action, module);

		res = fnc();

		if (res == 0) {
			ast_debug(3, "%s module " AST_MODULE "... (%s)... done.\n", action, module);
		}
		else {
			ast_debug(3, "%s module " AST_MODULE "... (%s)... failed(%d).\n", action, module, res);
		}
		return res;
	}

	int load_module2(const char *module, int (*fnc)())
	{
		return process_module("Loading", module, fnc);
	}

	int unload_module2(const char *module, int (*fnc)())
	{
		return process_module("Unloading", module, fnc);
	}

	int reload_module2(const char *module, int (*fnc)())
	{
		return process_module("Reloading", module, fnc);
	}

	static int load_module(void)
	{
		ast_debug(3, "Loading module " AST_MODULE "...\n");

		int res = 0;
		res |= load_module2("activecalls", load_module_activecalls);
		res |= load_module2("blacklist", load_module_blacklist);
		res |= load_module2("callconstraints", load_module_callconstraints);
		res |= load_module2("callpermissions", load_module_callpermissions);
		res |= load_module2("dialplan", load_module_dialplan);
		res |= load_module2("helper", load_module_helper);
		res |= load_module2("history", load_module_history);
		res |= load_module2("routing", load_module_routing);
		res |= load_module2("security", load_module_security);
		res |= load_module2("timerange", load_module_timerange);

		ast_debug(3, "Loading module " AST_MODULE "... %s.\n", (res == 0 ? "done" : "failed"));
		return res;
	}

	static int unload_module(void)
	{
		int res = 0;
		ast_debug(3, "Unloading module " AST_MODULE "...\n");

		res |= unload_module2("activecalls", unload_module_activecalls);
		res |= unload_module2("blacklist", unload_module_blacklist);
		res |= unload_module2("callconstraints", unload_module_callconstraints);
		res |= unload_module2("callpermissions", unload_module_callpermissions);
		res |= unload_module2("dialplan", unload_module_dialplan);
		res |= unload_module2("helper", unload_module_helper);
		res |= unload_module2("history", unload_module_history);
		res |= unload_module2("routing", unload_module_routing);
		res |= unload_module2("security", unload_module_security);
		res |= unload_module2("timerange", unload_module_timerange);

		ast_debug(3, "Unloading module " AST_MODULE "... %s.\n", (res == 0 ? "done" : "failed"));
		return res;
	}

	static int reload_module(void)
	{
		int res = 0;
		ast_debug(3, "Reloading module " AST_MODULE "...\n");

		res |= reload_module2("activecalls", reload_module_activecalls);
		res |= reload_module2("blacklist", reload_module_blacklist);
		res |= reload_module2("callconstraints", reload_module_callconstraints);
		res |= reload_module2("callpermissions", reload_module_callpermissions);
		res |= reload_module2("dialplan", reload_module_dialplan);
		res |= reload_module2("helper", reload_module_helper);
		res |= reload_module2("history", reload_module_history);
		res |= reload_module2("routing", reload_module_routing);
		res |= reload_module2("security", reload_module_security);
		res |= reload_module2("timerange", reload_module_timerange);

		ast_debug(3, "Reloading module " AST_MODULE "... %s.\n", (res == 0 ? "done" : "failed"));
		return 0;
	}

#ifdef	__cplusplus
}
#endif

/**
 * load/unload/reload functions registration to asterisk
 */
AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, AST_MODULE,
		.load = load_module,
		.unload = unload_module,
		.reload = reload_module
		);
