#include <RoutingInterface.h>

static char *routing_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio routing show";
			e->usage = "Usage: kerio routing show\n"
				"	Print routing\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	routing_print(a->fd);
	return CLI_SUCCESS;
}

static char *routing_cli_test(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio routing test";
			e->usage = "Usage: kerio routing test <incoming|incoming_rev|outgoing> <interface|routeId> <number>\n";
			return NULL;
		case CLI_GENERATE:
			if (a->pos == 3) {
				const char *choices[] = {"incoming", "incoming_rev", "outgoing", NULL};
				return ast_cli_complete(a->word, (const char**) choices, a->n);
			}
			return NULL;

		default:
			if (a->argc != e->args + 3) {
				return CLI_SHOWUSAGE;
			}
	}

	char buf[1024];
	int len = 1023;
	if (0 == strcmp(a->argv[3], "incoming")) {
		routing_find_incoming(buf, len, a->argv[4], a->argv[5]);
		ast_cli(a->fd, "Result: %s\n", buf);
	}
	if (0 == strcmp(a->argv[3], "incoming_rev")) {
		routing_find_incoming_rev(buf, len, a->argv[4], a->argv[5]);
		ast_cli(a->fd, "Result: %s\n", buf);
	}
	if (0 == strcmp(a->argv[3], "outgoing")) {

		routing_find_outgoing(buf, len, a->argv[4], a->argv[5]);
		ast_cli(a->fd, "Result: %s\n", buf);
	}

	return CLI_SUCCESS;
}

static int routing_func_incoming_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(interface);
		AST_APP_ARG(number);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_INCOMING(interface,number) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 2) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_INCOMING(interface,number) - missing arguments!\n");
		return -1;
	}

	routing_find_incoming(buf, len, args.interface, args.number);
	ast_verb(3, "Routing incoming result (interface=%s,number=%s) = [%s].\n", args.interface, args.number, buf);

	return 0;
}

static int routing_func_incoming_rev_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(interface);
		AST_APP_ARG(number);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_INCOMING_REV(interface,number) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 2) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_INCOMING_REV(interface,number) - missing arguments!\n");
		return -1;
	}


	routing_find_incoming_rev(buf, len, args.interface, args.number);
	ast_verb(3, "Routing incoming rev result (interface=%s,number=%s) = [%s].\n", args.interface, args.number, buf);

	return 0;
}

static int routing_func_outgoing_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(routeId);
		AST_APP_ARG(number);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_OUTGOING(routeId,number) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 2) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_OUTGOING(routeId,number) - missing arguments!\n");
		return -1;
	}


	routing_find_outgoing(buf, len, args.routeId, args.number);
	ast_verb(3, "Routing outgoing result (route=%s,number=%s) = [%s].\n", args.routeId, args.number, buf);

	return 0;
}

static int routing_func_hidden_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(routeId);
		AST_APP_ARG(number);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_HIDDEN(routeId,number) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 2) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_ROUTING_HIDDEN(routeId,number) - missing arguments!\n");
		return -1;
	}


	routing_hidden(buf, len, args.routeId, args.number);
	ast_verb(3, "Routing hidden result (route=%s,number=%s) = [%s].\n", args.routeId, args.number, buf);

	return 0;
}

static struct ast_custom_function routing_func_incoming = {
	.name = "KERIO_ROUTING_INCOMING",
	.desc = "Find external number in the routing table",
	.syntax = "KERIO_ROUTING_INCOMING(interface,number)",
	.read = routing_func_incoming_read,
};

static struct ast_custom_function routing_func_incoming_rev = {
	.name = "KERIO_ROUTING_INCOMING_REV",
	.desc = "Find internal number in the routing table",
	.syntax = "KERIO_ROUTING_INCOMING_REV(interface,number)",
	.read = routing_func_incoming_rev_read,
};


static struct ast_custom_function routing_func_outgoing = {
	.name = "KERIO_ROUTING_OUTGOING",
	.desc = "Find internal number in the routing table",
	.syntax = "KERIO_ROUTING_OUTGOING(routeId,number)",
	.read = routing_func_outgoing_read,
};

static struct ast_custom_function routing_func_hidden = {
	.name = "KERIO_ROUTING_HIDDEN",
	.desc = "Return 1 when callerid for number should be hidden",
	.syntax = "KERIO_ROUTING_HIDDEN(routeId,number)",
	.read = routing_func_hidden_read,
};


static struct ast_cli_entry routing_cli[] = {
	AST_CLI_DEFINE(routing_cli_show, "Print routing"),
	AST_CLI_DEFINE(routing_cli_test, "Test routing")
};

static int load_module_routing(void)
{
	routing_load();

	ast_cli_register_multiple(routing_cli, ARRAY_LEN(routing_cli));
	ast_custom_function_register(&routing_func_incoming);
	ast_custom_function_register(&routing_func_incoming_rev);
	ast_custom_function_register(&routing_func_outgoing);
	ast_custom_function_register(&routing_func_hidden);

	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module_routing(void)
{
	ast_cli_unregister_multiple(routing_cli, ARRAY_LEN(routing_cli));
	ast_custom_function_unregister(&routing_func_incoming);
	ast_custom_function_unregister(&routing_func_incoming_rev);
	ast_custom_function_unregister(&routing_func_outgoing);
	ast_custom_function_unregister(&routing_func_hidden);

	routing_unload();

	return 0;
}

static int reload_module_routing(void)
{
	routing_unload();
	routing_load();

	return 0;
}
