#include <module.h>
#include <SecurityInterface.h>

extern int kerio_security_loaded; // defined in asterisk's main/asterisk.c, used in chan_sip.c

static char *security_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio security show";
			e->usage = "Usage: kerio security show\n"
				"	Print current security status\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	security_print(a->fd);
	return CLI_SUCCESS;
}

static struct ast_cli_entry security_cli[] = {
	AST_CLI_DEFINE(security_cli_show, "Print current security status"),
};

static int load_module_security(void)
{
	security_load();

	ast_cli_register_multiple(security_cli, ARRAY_LEN(security_cli));

	kerio_security_loaded = 1;

	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module_security(void)
{
	kerio_security_loaded = 0;

	ast_cli_unregister_multiple(security_cli, ARRAY_LEN(security_cli));

	security_unload();

	return 0;
}

static int reload_module_security(void)
{
	kerio_security_loaded = 0;

	security_unload();
	security_load();

	kerio_security_loaded = 1;

	return 0;
}
